package glg203.serialization;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import glg203.serialization.observer.PersonObserver;
import glg203.serialization.observer.events.EventType;
import glg203.serialization.observer.events.PersonEvent;

/**
 * A group of person.
 * Is observable (pattern Observable/Observer) and Serializable.
 */
public class PersonGroup implements Serializable {
    // We don't save the observer list.
    private  transient List<PersonObserver> observers = new ArrayList<>();

    private Set<Person> personSet = new HashSet<>();

    public PersonGroup() {
        observers = new ArrayList<>();
    }

    public void addObserver(PersonObserver o) {
        observers.add(o);        
    }

    public void removeObserver(PersonObserver o) {
        observers.remove(o);        
    }


    public Person addPerson(String name, String address) {
        Person p = new Person(this, name, address);
        personSet.add(p);
        notifyModification(new PersonEvent(EventType.ADDED, p));
        return p;
    }

    public void removePerson(Person p) {
        if (personSet.contains(p)) {
            personSet.remove(p);
            p.clearOwner();
            notifyModification(new PersonEvent(EventType.REMOVED, p));
        }
    }

    public void notifyModification(PersonEvent personEvent) {
        for (PersonObserver personObserver : observers) {
            personObserver.personEventOccurred(personEvent);
        }
    }

    /**
     * Appelée lors de la désérialisation.
     * Sans elle, la liste d'observateurs (transient) est à null.
     * @return
     */
    private Object readResolve() {
        this.observers = new ArrayList<>();
        return this;
    }


    @Override
    public String toString() {
        return personSet.toString();
    }

}
