package demojdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;


public class Exemple0 {
    
    public static void main(String[] args) throws SQLException {
        Exemple0 m = new Exemple0();
        m.run();
    }

    // URL Pour H2. Sans DB_CLOSE_DELAY, la base en mémoire serait détruite après
    // fermeture...
    String url = "jdbc:h2:mem:demo;DB_CLOSE_DELAY=-1";

    // Version sauvée dans un fichier
    // String url = "jdbc:h2:~/demojdbc";

    String login = "sa";
    String password = "";


    private void run() throws SQLException {
        createDatabase();
        fillDatabase();
        queryDatabase();
    }

    private void queryDatabase() throws SQLException {
        try (
                Connection db = getConnection();
                Statement st = db.createStatement();) {
            String sql = "select * from person";
            try (ResultSet res = st.executeQuery(sql)) {
                while (res.next()) {
                    int id = res.getInt("id");
                    String name = res.getString("name");
                    System.out.printf("Person: %d name: %s\n", id, name);
                }
            }
        }
    }

    private void fillDatabase() throws SQLException {
        String names[] = {
                "joe",
                "jack",
                "william",
                "averell"
        };
        String sql = "insert into person (id, name) values (?,?)";
        try (
                Connection db = getConnection();
                PreparedStatement pst = db.prepareStatement(sql);) {
            for (int i = 0; i < names.length; i++) {
                pst.setInt(1, i + 1);
                pst.setString(2, names[i]);
                pst.executeUpdate();
            }
        }
    }

    private void createDatabase() throws SQLException {
        try (
                Connection db = getConnection();
                Statement st = db.createStatement();) {
            String sql = "create table person ("
                    + " id integer primary key not null,"
                    + " name varchar(30))";
            st.executeUpdate(sql);
        }
    }

    private Connection getConnection() throws SQLException {
        return DriverManager.getConnection(url, login, password);
    }
}
