package glg203;

import java.sql.Connection;
import java.sql.SQLException;

import glg203.dao.DAOException;
import glg203.dao.PersonDAO;
import glg203.db.ConnectionFactory;
import glg203.db.DatabaseCreator;
import glg203.model.Person;

public class Application {
 
    public static void main(String[] args) {
        new Application().run();
    }
    
    public void run() {
        prepareDB();
        // Le code suivant devrait se trouver dans la logique applicative :
        try (Connection db = new ConnectionFactory().getConnection()) {
            doWithConnection(db);
        } catch(SQLException e) { 
            e.printStackTrace(); 
        }    
    }

    private void doWithConnection(Connection db) {
        PersonDAO dao = new PersonDAO(db);
        int id1 = dao.create(new Person("toto"));
        System.out.println("On a créé "+ id1);

        int id2 = dao.create(new Person("titi"));
        System.out.println("On a créé "+ id2);

        System.out.println();
        System.out.println("Liste des personnes ");
        for (Person p : dao.findAll()) {
            System.out.println(p);
        }

        System.out.println("essai avec id2...");
        try {
            Person p2 = new Person(id2, "tutu");
            dao.create(p2);
        } catch (DAOException e) {
            System.out.println("Problème (normal) de création de cette entrée...");           
            System.out.println("Message "+e.getMessage()); 
        }
        System.out.println("suppression de id2.");
        dao.remove(id2);
        System.out.println("Liste des personnes sans numéro 2");
        for (Person p : dao.findAll()) {
            System.out.println(p);
        }
        System.out.println("Ajout d'un nouveau numéro 2");
        Person p2 = new Person(id2, "tutu");
        dao.create(p2);
        System.out.println("Liste des personnes à jour");
        for (Person p : dao.findAll()) {
            System.out.println(p);
        }
        System.out.println("Modification du numéro 2");
        p2.setId(2);
        p2.setName("Knuth");
        dao.update(p2);
        System.out.println("Liste finale des personnes");
        for (Person p : dao.findAll()) {
            System.out.println(p);
        }        
    }

    private void prepareDB() {
            try (Connection connection = new ConnectionFactory().getConnection()) {
                new DatabaseCreator().createDatabase(connection);
            } catch (SQLException e) {
               throw new RuntimeException(e); // bug du programme.
            }            
    }

}
