package glg203.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import glg203.model.Person;

public class PersonDAO {
    Connection connection;

    public PersonDAO(Connection connection) {
        this.connection = connection;
    }

    /**
     * Saves p in the database.
     * <p>
     * p must not already exist here (with the same id).
     * <p>
     * post : if p.getId() was null, p is modified and has an id.
     * 
     * @param p
     * @return
     * @throws DAOException if p is there
     */
    public int create(Person p) {
        try {
            if (p.getId() == null) {
                String sql = "insert into person (name) values (?)";
                try (PreparedStatement pst = connection.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS)) {
                    pst.setString(1, p.getName());
                    pst.executeUpdate();
                    try (ResultSet keyResultSet = pst.getGeneratedKeys();) {
                        if (keyResultSet.next()) {
                            return keyResultSet.getInt(1);
                        } else {
                            throw new DAOException("problem in generating key");
                        }
                    }
                }
            } else {
                String sql = "insert into person (id, name) values (?, ?)";
                try (PreparedStatement pst = connection.prepareStatement(sql)) {
                    pst.setInt(1, p.getId());
                    pst.setString(2, p.getName());
                    pst.executeUpdate();
                    return p.getId();
                }
            }
        } catch (SQLException e) {
            throw new DAOException(e);
        }
    }

    private Person extractPerson(ResultSet res) throws SQLException {
        int id = res.getInt("id");
        String name = res.getString("name");
        return new Person(id, name);
    }

    public Person findById(int id) {
        String sql = "select * from person where id = ?";
        try (PreparedStatement pst = connection.prepareStatement(sql)) {
            pst.setInt(1, id);
            try (ResultSet r = pst.executeQuery()) {
                if (r.next())
                    return extractPerson(r);
                else
                    return null;
            }
        } catch (SQLException e) {
            throw new DAOException(e);
        }
    }

    public Collection<Person> findAll() {
        try (
                Statement st = connection.createStatement();
                ResultSet rs = st.executeQuery("select * from Person")) {
            List<Person> result = new ArrayList<>();
            while (rs.next()) {
                result.add(extractPerson(rs));
            }
            return result;
        } catch (SQLException e) {
            throw new DAOException(e);
        }
    }

    public void remove(int id) {
        String sql = "delete from person where id = ?";
        try (PreparedStatement pst = connection.prepareStatement(sql)) {
            pst.setInt(1, id);
            pst.executeUpdate();
        } catch (SQLException e) {
            throw new DAOException(e);
        }
    }

    public void update(Person p) {
        String sql = "update person set name = ? where id = ?";
        try (PreparedStatement pst = connection.prepareStatement(sql)) {
            pst.setInt(2, p.getId());
            pst.setString(1, p.getName());
            pst.executeUpdate();
        } catch (SQLException e) {
            throw new DAOException(e);
        }
    }
}
