package glg203.exemplemysql;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class PersonneDAO {
    private final Connection db;

    public PersonneDAO(Connection db) {
        this.db = db;
    }

    public void save(Person p) {
        String sql = "insert into person values (?,?)";
        try (
                PreparedStatement pst = db.prepareStatement(sql);) {
            pst.setInt(1, p.getId());
            pst.setString(2, p.getName());
            pst.executeUpdate();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    public Person findById(int id) {
        String sql = "select * from person where id = ?";
        try (
                PreparedStatement pst = db.prepareStatement(sql);) {
            pst.setInt(1, id);
            try (ResultSet r = pst.executeQuery()) {
                if (r.next()) {
                    return extract(r);
                } else {
                    return null;
                }
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    public List<Person> findAll() {
        try (
                Statement pst = db.createStatement();
                ResultSet r = pst.executeQuery("select * from person")) {
            List<Person> result = new ArrayList<>();
            while (r.next()) {
                result.add(extract(r));
            }
            return result;
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    private Person extract(ResultSet res) {
        try {
            int id;
            id = res.getInt("id");

            String name = res.getString("name");
            return new Person(id, name);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

}
