# DAO et Template Method

Les DAO sont souvent répétitives. Sans aller jusqu'à réécrire JPA, peut-on factoriser et simplifier leur code ? Nous utilisons ici de manière assez systématique le pattern **Template Method** pour proposer une approche. 

Vous y échappez en TP, parce que ce serait devenu un TP sur ce mini-framework et pas sur le jdbc :-)

Avec les classes de `glg203.persistenceFramework`, l'écriture d'un objet en base devient quelque chose comme :

~~~java
    String sql = "insert into Person values (?,?)";        
    SQLSimpleHelper.update(getConnection(), sql, p.getId(), p.getName());
~~~

La récupération est un peu plus complexe, parce qu'il faut fournir une *lambda* qui va extraire les données du `ResultSet` pour retourner un objet :

~~~java
 String sql = "select * from person where id = ?";
 List<Person> result = SQLSimpleHelper.query(getConnection(), 
                                sql, resultSet -> extractPerson(resultSet),
                                id);            
~~~

## À propos des exceptions

Dans cet exemple, l'idée (qui devrait être complétée) est de séparer les exceptions *techniques* (du type : erreur de syntaxe SQL) qui sont des bugs, et devraient être des RuntimeException des exceptions logiques (clef dupliquée, violation de contraintes, etc...) qui proviennent potentiellement de saisies utilisateur.

Nous ne l'avons réellement fait que sur l'erreur DuplicateKeyException et pour quelques contraintes d'intégrité (partiellement).

Les codes d'erreur sont spécifiques au serveur de SGBD utilisé, ce qui est éventuellement problématique.
Pour notre application, on pourrait envisager une factory method prenant comme argument une erreur SQL et renvoyant :
- soit une `DAOException` si l'erreur semble liée à la logique du logiciel ;
- soit une `TechnicalSQLException` si c'est a priori un bug.

