package glg203;

import java.sql.Connection;
import java.sql.SQLException;

import glg203.dao.PersonDAO;
import glg203.db.ConnectionFactory;
import glg203.db.DatabaseCreator;
import glg203.model.Person;
import glg203.persistenceFramework.exception.CheckException;
import glg203.persistenceFramework.exception.DAOException;
import glg203.persistenceFramework.exception.DuplicateKeyException;

public class Application {

    public void run() {
        prepareDB();
        // Le code suivant devrait se trouver dans la logique applicative :
        try (Connection db = new ConnectionFactory().getConnection()) {
            doWithConnection(db);
        } catch (SQLException e) { // et ce code devrait être dans la couuche de persistance...
            e.printStackTrace();
        }
    }

    private void doWithConnection(Connection db) {
        try {
            PersonDAO dao = new PersonDAO(db);
            dao.save(new Person(1l, "toto"));

            dao.save(new Person(2l, "titi"));

            println();
            println("Liste des personnes ");
            for (Person p : dao.findAll()) {
                println(p);
            }

            println("essai avec id2...");
            try {
                Person p2 = new Person(2l, "tutu");
                dao.save(p2);
            } catch (DuplicateKeyException e) {
                println("Problème (normal) de création de cette entrée...");
                println("Message " + e.getMessage());
            }
            println("suppression de id2.");
            dao.remove(2l);
            println("Liste des personnes sans numéro 2");
            for (Person p : dao.findAll()) {
                println(p);
            }
            println("Ajout d'un nouveau numéro 2");
            Person p2 = new Person(2l, "tutu");
            dao.save(p2);
            println("Liste des personnes à jour");
            for (Person p : dao.findAll()) {
                println(p);
            }
            println("Modification du numéro 2");
            p2.setId(2l);
            p2.setName("Knuth");
            dao.update(p2);
            println("Liste finale des personnes");
            for (Person p : dao.findAll()) {
                println(p);
            }
            println("Essai de violation de contraintes:");
            try {
                dao.save(new Person(100l, null));
            } catch (CheckException e) {
                println("Contraintes d'intégrité non respectées");
            }
        } catch (DAOException e) {
            println("Erreur inattendue");
            e.printStackTrace();
        }
    }

    private void prepareDB() {
        try (Connection connection = new ConnectionFactory().getConnection()) {
            new DatabaseCreator().createDatabase(connection);
        } catch (SQLException e) {
            throw new RuntimeException(e); // bug du programme.
        }
    }

    private void println() {
        System.out.println();
    }

    private void println(Object o) {
        System.out.println(o);
    }

    public static void main(String[] args) {
        new Application().run();
    }
}
