package glg203.persistenceFramework;

import java.sql.Connection;
import java.util.List;

import glg203.persistenceFramework.exception.CheckException;
import glg203.persistenceFramework.exception.DuplicateKeyException;
import glg203.persistenceFramework.exception.NoSuchElementException;
/**
 * Une base pour implémenter le pattern Data Access Object.
 * Dans cette version, les identifiants sont supposés être systématiquement des
 * Strings.
 */
public abstract class AbstractDAO<T extends DomainObject> {

    private Connection connection;

    protected AbstractDAO(Connection connection) {
        this.connection = connection;
    }

    protected Connection getConnection() {
        return connection;
    }

    /**
     * Sauvegarde un objet en base.
     * 
     * @param obj
     */
    public abstract void save(T obj)  throws DuplicateKeyException, CheckException;

    /**
     * Met à jour un objet déjà connu.
     * 
     * @param obj
     */
    public abstract void update(T obj) throws NoSuchElementException, CheckException;

    /**
     * Détruit un objet dont on passe l'identifiant.
     * 
     * @param obj
     */
    public abstract void remove(Long id) throws NoSuchElementException, CheckException;

    /**
     * Cherche un objet par identifiant.
     * 
     * @param obj
     */
    public abstract T findById(Long id);

    /**
     * Renvoie tous les objets.
     * 
     * @param obj
     */
    public abstract List<T> findAll();
    
}
