package glg203.persistenceFramework.template;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Requête SQL qui retourne un résultat de type T.
 * 
 * @param T le type du résultat.
 */
class SQLQueryTemplate<T> extends SQLStatementTemplate<List<T>>{
    ResultSetExtractor<T> extractor;


    public SQLQueryTemplate(Connection connection, String sql, ResultSetExtractor<T> extractor) {
        super(connection, sql);
        this.extractor = extractor;
    }


    @Override
    protected final List<T> performWithStatement(PreparedStatement pst) throws SQLException {
        ArrayList<T> result = new ArrayList<>();
        try (ResultSet resultSet = pst.executeQuery()) {
            while (resultSet.next()) {
                result.add(extractor.extract(resultSet));
            }
        }
        return result;
    }
}